local table = require("hp/lang/table")

--------------------------------------------------------------------------------
-- クラスベースなオブジェクト指向を簡単に実現するためのクラスです.<br>
-- クラスの基本的な機能を有します.<br>
-- setmetatableによる継承ではなく、テーブルに展開する事で<br>
-- 継承が多くなった場合でもパフォーマンスが劣化しません.<br>
-- @class table
-- @name class
--------------------------------------------------------------------------------
local M = {}
setmetatable(M, M)

--------------------------------------------------------------------------------
-- クラス定義関数です.
-- 引数にテーブルを指定する事で多重継承が可能です.
-- @param ... 親テーブルです. 多重継承が可能です.
-- @return class
--------------------------------------------------------------------------------
function M:__call(...)
    local class = table.copy(self)
    for i, super in ipairs({...}) do
        table.copy(super, class)
    end
    class.__call = nil
    return class
end

--------------------------------------------------------------------------------
-- コンストラクタ関数です.
-- デフォルトではテーブルを生成して、メタテーブルによる参照を設定します.
-- @return インスタンス
--------------------------------------------------------------------------------
function M:new(...)
    local obj = {__index = self}
    setmetatable(obj, obj)
    
    if obj.init then
        obj:init(...)
    end

    obj.new = nil
    obj.init = nil
    
    return obj
end

return M
